@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,7 +20,7 @@
 """
 
 template_constants_reading = """
-#include "nuitka/prelude.hpp"
+#include "nuitka/prelude.h"
 
 // Sentinel PyObject to be used for all our call iterator endings. It will
 // become a PyCObject pointing to NULL. It's address is unique, and that's
@@ -29,25 +29,6 @@ PyObject *_sentinel_value = NULL;
 
 %(constant_declarations)s
 
-#if defined(_WIN32) && defined(_NUITKA_EXE)
-#include <Windows.h>
-const unsigned char* constant_bin;
-struct __initResourceConstants
-{
-    __initResourceConstants()
-    {
-        constant_bin = (const unsigned char*)LockResource(
-            LoadResource(
-                NULL,
-                FindResource(NULL, MAKEINTRESOURCE(3), RT_RCDATA)
-            )
-        );
-    }
-} __initResourceConstants_static_initializer;
-#else
-extern "C" const unsigned char constant_bin[];
-#endif
-
 static void _createGlobalConstants( void )
 {
     NUITKA_MAY_BE_UNUSED PyObject *exception_type, *exception_value;
@@ -60,6 +41,14 @@ static void _createGlobalConstants( void )
 #endif
 
 %(constant_inits)s
+
+#if _NUITKA_EXE
+    /* Set the "sys.executable" path to the original CPython executable. */
+    PySys_SetObject(
+        (char *)"executable",
+        %(sys_executable)s
+    );
+#endif
 }
 
 // In debug mode we can check that the constants were not tampered with in any
@@ -72,7 +61,6 @@ void checkGlobalConstants( void )
 }
 #endif
 
-
 void createGlobalConstants( void )
 {
     if ( _sentinel_value == NULL )
