@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,7 +18,7 @@
 """ The type1 node.
 
 This one just determines types. It's great for optimization. We may be able to
-predict its value, but knowing it. In that case, we have a builtin name
+predict its value, but knowing it. In that case, we have a built-in name
 reference for that type to convert to, or when checking the result of it, we
 will then know it's limited after the fact.
 
@@ -26,6 +26,10 @@ will then know it's limited after the fact.
 
 from nuitka.Builtins import builtin_names
 
+from .BuiltinRefNodes import (
+    ExpressionBuiltinAnonymousRef,
+    ExpressionBuiltinRef
+)
 from .NodeBases import (
     ExpressionBuiltinSingleArgBase,
     ExpressionChildrenHavingBase
@@ -35,7 +39,7 @@ from .NodeBases import (
 class ExpressionBuiltinType1(ExpressionBuiltinSingleArgBase):
     kind = "EXPRESSION_BUILTIN_TYPE1"
 
-    def computeExpression(self, constraint_collection):
+    def computeExpression(self, trace_collection):
         value = self.getValue()
 
         if value.isCompileTimeConstant():
@@ -43,10 +47,6 @@ class ExpressionBuiltinType1(ExpressionBuiltinSingleArgBase):
 
             type_name = value.__class__.__name__
 
-            from .BuiltinRefNodes import (
-                ExpressionBuiltinAnonymousRef,
-                ExpressionBuiltinRef
-            )
 
             if type_name in builtin_names:
                 new_node = ExpressionBuiltinRef(
@@ -69,7 +69,7 @@ class ExpressionBuiltinType1(ExpressionBuiltinSingleArgBase):
 
         return self, None, None
 
-    def computeExpressionDrop(self, statement, constraint_collection):
+    def computeExpressionDrop(self, statement, trace_collection):
         from .NodeMakingHelpers import \
           makeStatementExpressionOnlyReplacementNode
 
@@ -81,6 +81,12 @@ class ExpressionBuiltinType1(ExpressionBuiltinSingleArgBase):
         return result, "new_statements", """\
 Removed type taking for unused result."""
 
+    def mayRaiseException(self, exception_type):
+        return self.getValue().mayRaiseException(exception_type)
+
+    def mayHaveSideEffects(self):
+        return self.getValue().mayHaveSideEffects()
+
 
 class ExpressionBuiltinSuper(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_SUPER"
@@ -103,7 +109,9 @@ class ExpressionBuiltinSuper(ExpressionChildrenHavingBase):
     getType = ExpressionChildrenHavingBase.childGetter("type")
     getObject = ExpressionChildrenHavingBase.childGetter("object")
 
-    def computeExpression(self, constraint_collection):
+    def computeExpression(self, trace_collection):
+        trace_collection.onExceptionRaiseExit(BaseException)
+
         # TODO: Quite some cases should be possible to predict.
         return self, None, None
 
@@ -124,11 +132,37 @@ class ExpressionBuiltinIsinstance(ExpressionChildrenHavingBase):
                 "classes"  : classes
 
             },
-            source_ref = source_ref )
+            source_ref = source_ref
+        )
 
     getInstance = ExpressionChildrenHavingBase.childGetter("instance")
     getCls = ExpressionChildrenHavingBase.childGetter("classes")
 
-    def computeExpression(self, constraint_collection):
+    def computeExpression(self, trace_collection):
         # TODO: Quite some cases should be possible to predict.
-        return self, None, None
+
+        instance = self.getInstance()
+
+        # TODO: Should be possible to query run time type instead, but we don't
+        # have that method yet. Later this will be essential.
+        if not instance.isCompileTimeConstant():
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        cls = self.getCls()
+
+        if not cls.isCompileTimeConstant():
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        # So if both are compile time constant, we are able to compute it.
+        return trace_collection.getCompileTimeComputationResult(
+            node        = self,
+            computation = lambda : isinstance(
+                instance.getCompileTimeConstant(),
+                cls.getCompileTimeConstant()
+            ),
+            description = "Built-in call to 'isinstance' computed."
+        )
