@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -49,11 +49,9 @@ catching and passing in exceptions raised.
 
 """
 
-
-import ast
 import sys
 
-from nuitka import Options, PythonVersions, SourceCodeReferences, Tracing
+from nuitka import Options, SourceCodeReferences, Tracing
 from nuitka.__past__ import long, unicode  # pylint: disable=W0622
 from nuitka.importing import Importing
 from nuitka.importing.ImportCache import addImportedModule
@@ -63,17 +61,28 @@ from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable
 )
 from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
+from nuitka.nodes.BuiltinFormatNodes import (
+    ExpressionBuiltinAscii,
+    ExpressionBuiltinFormat
+)
+from nuitka.nodes.BuiltinTypeNodes import ExpressionBuiltinStr
 from nuitka.nodes.ConditionalNodes import (
     ExpressionConditional,
     StatementConditional
 )
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import (
+    ExpressionConstantEllipsisRef,
+    ExpressionConstantNoneRef,
+    makeConstantRefNode
+)
+from nuitka.nodes.CoroutineNodes import ExpressionAsyncWait
 from nuitka.nodes.ExceptionNodes import StatementRaiseException
+from nuitka.nodes.GeneratorNodes import StatementGeneratorReturn
 from nuitka.nodes.ImportNodes import (
     ExpressionImportModule,
     ExpressionImportName
 )
-from nuitka.nodes.LoopNodes import StatementBreakLoop, StatementContinueLoop
+from nuitka.nodes.LoopNodes import StatementLoopBreak, StatementLoopContinue
 from nuitka.nodes.ModuleNodes import (
     CompiledPythonModule,
     CompiledPythonPackage,
@@ -82,18 +91,26 @@ from nuitka.nodes.ModuleNodes import (
     PythonShlibModule
 )
 from nuitka.nodes.OperatorNodes import (
-    ExpressionOperationBinary,
-    ExpressionOperationUnary
+    ExpressionOperationUnary,
+    makeBinaryOperationNode
 )
-from nuitka.nodes.ReturnNodes import ExpressionAwait, StatementReturn
+from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.StatementNodes import StatementExpressionOnly
+from nuitka.nodes.StringConcatenationNodes import ExpressionStringConcatenation
 from nuitka.nodes.VariableRefNodes import ExpressionVariableRef
+from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
-from nuitka.utils import Utils
+from nuitka.tree.ReformulationForLoopStatements import (
+    buildAsyncForLoopNode,
+    buildForLoopNode
+)
+from nuitka.tree.ReformulationWhileLoopStatements import buildWhileLoopNode
+from nuitka.utils import MemoryUsage, Utils
 
 from . import SyntaxErrors
 from .Helpers import (
     buildNode,
+    buildNodeList,
     buildStatementsNode,
     extractDocFromBody,
     getBuildContext,
@@ -104,6 +121,7 @@ from .Helpers import (
     makeStatementsSequenceOrStatement,
     mangleName,
     mergeStatements,
+    parseSourceCodeToAst,
     setBuildingDispatchers
 )
 from .ReformulationAssertStatements import buildAssertNode
@@ -133,7 +151,6 @@ from .ReformulationImportStatements import (
     checkFutureImportsOnlyAtStart
 )
 from .ReformulationLambdaExpressions import buildLambdaNode
-from .ReformulationLoopStatements import buildForLoopNode, buildWhileLoopNode
 from .ReformulationNamespacePackages import (
     createNamespacePackage,
     createPathAssignment
@@ -143,28 +160,22 @@ from .ReformulationSequenceCreation import buildSequenceCreationNode
 from .ReformulationSubscriptExpressions import buildSubscriptNode
 from .ReformulationTryExceptStatements import buildTryExceptionNode
 from .ReformulationTryFinallyStatements import buildTryFinallyNode
-from .ReformulationWithStatements import buildWithNode
+from .ReformulationWithStatements import buildAsyncWithNode, buildWithNode
 from .ReformulationYieldExpressions import buildYieldFromNode, buildYieldNode
 from .SourceReading import readSourceCodeFromFilename
 from .VariableClosure import completeVariableClosures
 
 
 def buildVariableReferenceNode(provider, node, source_ref):
-    # Python3 is influenced by the mere use of a variable name. So we need to
-    # remember it, esp. for cases, where it is optimized away.
-    if python_version >= 300 and \
-       node.id == "super" and \
-       provider.isExpressionFunctionBody():
-        provider.markAsClassClosureTaker()
-
     return ExpressionVariableRef(
         variable_name = mangleName(node.id, provider),
         source_ref    = source_ref
     )
 
-# Python3.4 only, True and False, are not given as variables anymore.
+
+# Python3.4 or higher, True and False, are not given as variables anymore.
 def buildNamedConstantNode(node, source_ref):
-    return ExpressionConstantRef(
+    return makeConstantRefNode(
         constant   = node.value,
         source_ref = source_ref
     )
@@ -339,10 +350,10 @@ def buildImportModulesNode(provider, node, source_ref):
             StatementAssignmentVariable(
                 variable_ref = ExpressionTargetVariableRef(
                     variable_name = mangleName(
-                                      local_name
-                                        if local_name is not None else
-                                      module_topname,
-                                      provider
+                        local_name
+                          if local_name is not None else
+                        module_topname,
+                        provider
                     ),
                     source_ref    = source_ref
                 ),
@@ -369,7 +380,7 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
 
     # Need to catch the error of declaring a parameter variable as global
     # ourselves here. The AST parsing doesn't catch it, so we check here.
-    try:
+    if provider.isExpressionFunctionBody():
         parameters = provider.getParameters()
 
         for variable_name in node.names:
@@ -388,8 +399,6 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
                         provider.getSourceReference()
                     )
                 )
-    except AttributeError:
-        pass
 
     # The module the "global" statement refers to.
     module = provider.getParentModule()
@@ -438,10 +447,19 @@ def handleGlobalDeclarationNode(provider, node, source_ref):
 def handleNonlocalDeclarationNode(provider, node, source_ref):
     # Need to catch the error of declaring a parameter variable as global
     # ourselves here. The AST parsing doesn't catch it, but we can do it here.
-    parameters = provider.getParameters()
+    parameter_provider = provider
+
+    while parameter_provider.isExpressionGeneratorObjectBody() or \
+          parameter_provider.isExpressionCoroutineObjectBody():
+        parameter_provider = parameter_provider.getParentVariableProvider()
+
+    if parameter_provider.isExpressionClassBody():
+        parameter_names = ()
+    else:
+        parameter_names = parameter_provider.getParameters().getParameterNames()
 
     for variable_name in node.names:
-        if variable_name in parameters.getParameterNames():
+        if variable_name in parameter_names:
             SyntaxErrors.raiseSyntaxError(
                 reason       = "name '%s' is parameter and nonlocal" % (
                     variable_name
@@ -464,7 +482,7 @@ def handleNonlocalDeclarationNode(provider, node, source_ref):
 def buildStringNode(node, source_ref):
     assert type(node.s) in (str, unicode)
 
-    return ExpressionConstantRef(
+    return makeConstantRefNode(
         constant      = node.s,
         source_ref    = source_ref,
         user_provided = True
@@ -474,7 +492,7 @@ def buildStringNode(node, source_ref):
 def buildNumberNode(node, source_ref):
     assert type(node.n) in (int, long, float, complex), type(node.n)
 
-    return ExpressionConstantRef(
+    return makeConstantRefNode(
         constant      = node.n,
         source_ref    = source_ref,
         user_provided = True
@@ -482,7 +500,7 @@ def buildNumberNode(node, source_ref):
 
 
 def buildBytesNode(node, source_ref):
-    return ExpressionConstantRef(
+    return makeConstantRefNode(
         constant      = node.s,
         source_ref    = source_ref,
         user_provided = True
@@ -490,14 +508,13 @@ def buildBytesNode(node, source_ref):
 
 
 def buildEllipsisNode(source_ref):
-    return ExpressionConstantRef(
-        constant      = Ellipsis,
+    return ExpressionConstantEllipsisRef(
         source_ref    = source_ref,
         user_provided = True
     )
 
 
-def buildStatementContinueLoop(node, source_ref):
+def buildStatementLoopContinue(node, source_ref):
     # Python forbids this, although technically it's probably not much of
     # an issue.
     if getBuildContext() == "finally":
@@ -518,16 +535,16 @@ def buildStatementContinueLoop(node, source_ref):
             source_line = source_line
         )
 
-    return StatementContinueLoop(
+    return StatementLoopContinue(
         source_ref = source_ref
     )
 
 
-def buildStatementBreakLoop(provider, node, source_ref):
+def buildStatementLoopBreak(provider, node, source_ref):
     # A bit unusual, we need the provider, but not the node,
     # pylint: disable=W0613
 
-    return StatementBreakLoop(
+    return StatementLoopBreak(
         source_ref = source_ref
     )
 
@@ -554,17 +571,29 @@ def buildReturnNode(provider, node, source_ref):
 
     expression = buildNode(provider, node.value, source_ref, allow_none = True)
 
+    if provider.isExpressionGeneratorObjectBody():
+        if expression is not None and python_version < 330:
+            SyntaxErrors.raiseSyntaxError(
+                "'return' with argument inside generator",
+                source_ref = source_ref,
+            )
+
     if expression is None:
-        expression = ExpressionConstantRef(
-            constant      = None,
+        expression = ExpressionConstantNoneRef(
             source_ref    = source_ref,
             user_provided = True
         )
 
-    return StatementReturn(
-        expression = expression,
-        source_ref = source_ref
-    )
+    if provider.isExpressionGeneratorObjectBody():
+        return StatementGeneratorReturn(
+            expression = expression,
+            source_ref = source_ref
+        )
+    else:
+        return StatementReturn(
+            expression = expression,
+            source_ref = source_ref
+        )
 
 
 def buildExprOnlyNode(provider, node, source_ref):
@@ -604,7 +633,7 @@ def buildBinaryOpNode(provider, node, source_ref):
     left       = buildNode(provider, node.left, source_ref)
     right      = buildNode(provider, node.right, source_ref)
 
-    result = ExpressionOperationBinary(
+    result = makeBinaryOperationNode(
         operator   = operator,
         left       = left,
         right      = right,
@@ -634,12 +663,57 @@ def buildConditionalExpressionNode(provider, node, source_ref):
         source_ref     = source_ref
     )
 
+
 def buildAwaitNode(provider, node, source_ref):
-    return ExpressionAwait(
+    return ExpressionAsyncWait(
         expression = buildNode(provider, node.value, source_ref),
         source_ref = source_ref
     )
 
+
+def buildFormattedValueNode(provider, node, source_ref):
+    value = buildNode(provider, node.value, source_ref)
+
+    conversion = node.conversion % 4 if node.conversion > 0 else 0
+
+    if conversion == 0:
+        pass
+    elif conversion == 3:
+        value = ExpressionBuiltinStr(
+            value      = value,
+            encoding   = None,
+            errors     = None,
+            source_ref = source_ref
+        )
+    elif conversion == 2:
+        value = ExpressionOperationUnary(
+            operator   = "Repr",
+            operand    = value,
+            source_ref = source_ref
+        )
+    elif conversion == 1:
+        value = ExpressionBuiltinAscii(
+            value      = value,
+            source_ref = source_ref
+        )
+    else:
+        assert False, conversion
+
+    return ExpressionBuiltinFormat(
+        value       = value,
+        format_spec = buildNode(provider, node.format_spec, source_ref, allow_none = True),
+        source_ref  = source_ref
+    )
+
+
+def buildJoinedStrNode(provider, node, source_ref):
+    return ExpressionStringConcatenation(
+        values     = buildNodeList(provider, node.values, source_ref),
+        source_ref = source_ref
+    )
+
+
+
 setBuildingDispatchers(
     path_args3 = {
         "Name"              : buildVariableReferenceNode,
@@ -650,6 +724,7 @@ setBuildingDispatchers(
         "If"                : buildConditionNode,
         "While"             : buildWhileLoopNode,
         "For"               : buildForLoopNode,
+        "AsyncFor"          : buildAsyncForLoopNode,
         "Compare"           : buildComparisonNode,
         "ListComp"          : buildListContractionNode,
         "DictComp"          : buildDictContractionNode,
@@ -669,6 +744,7 @@ setBuildingDispatchers(
         "Assert"            : buildAssertNode,
         "Exec"              : buildExecNode,
         "With"              : buildWithNode,
+        "AsyncWith"         : buildAsyncWithNode,
         "FunctionDef"       : buildFunctionNode,
         "AsyncFunctionDef"  : buildAsyncFunctionNode,
         "Await"             : buildAwaitNode,
@@ -687,14 +763,16 @@ setBuildingDispatchers(
         "Repr"              : buildReprNode,
         "AugAssign"         : buildInplaceAssignNode,
         "IfExp"             : buildConditionalExpressionNode,
-        "Break"             : buildStatementBreakLoop,
+        "Break"             : buildStatementLoopBreak,
+        "JoinedStr"         : buildJoinedStrNode,
+        "FormattedValue"    : buildFormattedValueNode,
     },
     path_args2 = {
         "NameConstant" : buildNamedConstantNode,
         "Str"          : buildStringNode,
         "Num"          : buildNumberNode,
         "Bytes"        : buildBytesNode,
-        "Continue"     : buildStatementContinueLoop,
+        "Continue"     : buildStatementLoopContinue,
     },
     path_args1 = {
         "Ellipsis"     : buildEllipsisNode,
@@ -702,52 +780,15 @@ setBuildingDispatchers(
 )
 
 
-def _makeSyntaxErrorCompatible(e):
-    # Encoding problems for Python happen here, for Python3, this was
-    # already done when we read the source code.
-    if Options.isFullCompat() and \
-       (e.args[0].startswith("unknown encoding:") or \
-        e.args[0].startswith("encoding problem:")):
-        if PythonVersions.doShowUnknownEncodingName():
-            complaint = e.args[0].split(':',2)[1]
-        else:
-            complaint = " with BOM"
-
-        e.args = (
-            "encoding problem:%s" % complaint,
-            (e.args[1][0], 1, None, None)
-        )
-
-        if hasattr(e, "msg"):
-            e.msg = e.args[0]
-
-
-
 def buildParseTree(provider, source_code, source_ref, is_module, is_main):
     # There are a bunch of branches here, mostly to deal with version
-    # differences for module default variables. pylint: disable=R0912
-
-    # Workaround: ast.parse cannot cope with some situations where a file is not
-    # terminated by a new line.
-    if not source_code.endswith('\n'):
-        source_code = source_code + '\n'
-
-    try:
-        body = ast.parse(source_code, source_ref.getFilename())
-    except SyntaxError as e:
-        _makeSyntaxErrorCompatible(e)
-
-        raise e
-
-    assert getKind(body) == "Module"
-
-    line_offset = source_ref.getLineNumber() - 1
-
-    if line_offset > 0:
-        for created_node in ast.walk(body):
-            if hasattr(created_node, "lineno"):
-                created_node.lineno += line_offset
+    # differences for module default variables.
 
+    body = parseSourceCodeToAst(
+        source_code = source_code,
+        filename    = source_ref.getFilename(),
+        line_offset = source_ref.getLineNumber() - 1
+    )
     body, doc = extractDocFromBody(body)
 
     result = buildStatementsNode(
@@ -797,7 +838,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                     variable_name = "__doc__",
                     source_ref    = internal_source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = doc,
                     source_ref    = internal_source_ref,
                     user_provided = True
@@ -832,8 +873,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                     variable_name = "__cached__",
                     source_ref    = internal_source_ref
                 ),
-                source       = ExpressionConstantRef(
-                    constant      = None,
+                source       = ExpressionConstantNoneRef(
                     source_ref    = internal_source_ref,
                     user_provided = True
                 ),
@@ -850,7 +890,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                     variable_name = "__package__",
                     source_ref    = internal_source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = provider.getFullName()
                                       if provider.isCompiledPythonPackage() else
                                     provider.getPackage(),
@@ -872,7 +912,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                     variable_name = "__initializing__",
                     source_ref    = internal_source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = True,
                     source_ref    = internal_source_ref,
                     user_provided = True
@@ -881,6 +921,24 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
             )
         )
 
+    if python_version >= 360:
+        # Set "__annotations__" on module level to {}
+        statements.append(
+            StatementAssignmentVariable(
+                variable_ref = ExpressionTargetVariableRef(
+                    variable_name = "__annotations__",
+                    source_ref    = internal_source_ref
+                ),
+                source       = makeConstantRefNode(
+                    constant      = {},
+                    source_ref    = internal_source_ref,
+                    user_provided = True
+                ),
+                source_ref   = internal_source_ref
+            )
+        )
+
+
     # Now the module body if there is any at all.
     if result is not None:
         statements.extend(
@@ -895,7 +953,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
                     variable_name = "__initializing__",
                     source_ref    = internal_source_ref
                 ),
-                source       = ExpressionConstantRef(
+                source       = makeConstantRefNode(
                     constant      = False,
                     source_ref    = internal_source_ref,
                     user_provided = True
@@ -916,7 +974,7 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
 
 
 def decideModuleTree(filename, package, is_shlib, is_top, is_main):
-    # Many variables, branches, due to the many cases, pylint: disable=R0912
+    # Many variables, branches, due to the many cases, pylint: disable=R0912,R0915
 
     assert package is None or type(package) is str
     assert filename is not None
@@ -975,14 +1033,22 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
         elif is_main:
             result = PythonMainModule(
                 main_added = main_added,
+                mode       = Plugins.decideCompilation("__main__", source_ref),
                 source_ref = source_ref
             )
         else:
+            if package is not None:
+                full_name = package + '.' + module_name
+            else:
+                full_name = module_name
+
             result = CompiledPythonModule(
                 name         = module_name,
                 package_name = package,
+                mode         = Plugins.decideCompilation(full_name, source_ref),
                 source_ref   = source_ref
             )
+
     elif Importing.isPackageDir(filename):
         if is_top:
             package_name = Utils.splitpath(filename)[-1]
@@ -1002,9 +1068,15 @@ def decideModuleTree(filename, package, is_shlib, is_top, is_main):
                 filename = Utils.abspath(source_filename),
             )
 
+            if package is not None:
+                full_name = package + '.' + package_name
+            else:
+                full_name = package_name
+
             result = CompiledPythonPackage(
                 name         = package_name,
                 package_name = package,
+                mode         = Plugins.decideCompilation(full_name, source_ref),
                 source_ref   = source_ref
             )
     else:
@@ -1035,8 +1107,8 @@ class CodeTooComplexCode(Exception):
 
 
 def createModuleTree(module, source_ref, source_code, is_main):
-    if Options.isShowProgress():
-        memory_watch = Utils.MemoryWatch()
+    if Options.isShowMemory():
+        memory_watch = MemoryUsage.MemoryWatch()
 
     try:
         module_body = buildParseTree(
@@ -1064,7 +1136,7 @@ def createModuleTree(module, source_ref, source_code, is_main):
 
     completeVariableClosures(module)
 
-    if Options.isShowProgress():
+    if Options.isShowMemory():
         memory_watch.finish()
 
         Tracing.printLine(
