@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -33,7 +33,7 @@ def functionEval2():
 
     r = eval("1+3", g)
 
-    return r, g.keys()
+    return r, g.keys(), a
 
 print "eval in a function with globals provided", functionEval2()
 
@@ -49,7 +49,7 @@ print "eval in a for loop as iterator giver", functionEval3()
 
 print "exec on a global level",
 exec( "d=2+2" )
-print "2+2=",d
+print "2+2=",d  # @UndefinedVariable
 
 def functionExec1():
     a = 1
@@ -77,7 +77,7 @@ f.close()
 
 execfile(tmp_filename)
 
-print "execfile with defaults f,g=", e, f
+print "execfile with defaults f,g=", e, f  # @UndefinedVariable
 
 global_vars = { 'e' : '0', 'f' : 0 }
 local_vars = dict(global_vars)
@@ -251,7 +251,7 @@ def functionWithClosureProvidedByExec():
 
     exec code in None, None
 
-    def func( ):
+    def func():
         print "Closure from exec not used", ValueError
 
     func()
@@ -271,7 +271,7 @@ def functionWithExecAffectingClosure():
     def closureMaker():
         return x
 
-    return d, closureMaker()
+    return d, closureMaker()  # @UndefinedVariable
 
 print "Closure in a function with exec to not none", functionWithExecAffectingClosure()
 
@@ -303,10 +303,25 @@ def makeAddPair(a, b):
 """ in locals()
 
     if sys.version_info < (3,):
-        assert makeAddPair
+        assert makeAddPair  # @UndefinedVariable
 
     return "yes"
 
 print "Exec adds functions declares in explicit locals() given.", execDefinesFunctionToLocalsExplicity()
 
 os.unlink(tmp_filename)
+
+
+def execWithShortTuple():
+    try:
+        exec("print hey",)
+    except Exception as e:
+        return "gives exception: " + repr(e)
+
+print "Exec with too short tuple argument:", execWithShortTuple()
+
+if str is not bytes:
+    def evalMemoryView(value):
+        return eval(memoryview(value))
+
+    print "Eval with memory view:", evalMemoryView(b"27")
