@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -20,6 +20,23 @@ from __future__ import print_function
 
 import inspect, types, sys, pprint
 
+def displayDict(d, remove_keys = ()):
+    if "__loader__" in d:
+        d = dict(d)
+        d["__loader__"] = "<__loader__ removed>"
+
+    if "__file__" in d:
+        d = dict(d)
+        d["__file__"] = "<__file__ removed>"
+
+    for remove_key in remove_keys:
+        if remove_key in d:
+            d = dict(d)
+            del d[remove_key]
+
+    return pprint.pformat(d)
+
+
 def compiledFunction(a, b):
     pass
 
@@ -83,7 +100,7 @@ def someFunction(a):
 someFunction(2)
 
 class C:
-    print("Class locals", str(sys._getframe().f_locals).replace(", '__locals__': {...}", "").replace("'__qualname__': 'C', ", ""))
+    print("Class locals", displayDict(sys._getframe().f_locals, remove_keys = ("__qualname__","__locals__")))
     print("Class flags", sys._getframe().f_code.co_flags)
 
 def f():
@@ -92,16 +109,14 @@ def f():
 
 f()
 
-def displayDict(d):
-    if "__loader__" in d:
-        d = dict(d)
-        d["__loader__"] = "<__loader__ removed>"
+def g():
+    yield("Generator object locals", sys._getframe().f_locals)
+    yield("Generator object flags", sys._getframe().f_code.co_flags)
 
-    if "__file__" in d:
-        d = dict(d)
-        d["__file__"] = "<__file__ removed>"
+for line in g():
+    print(*line)
 
-    return pprint.pformat(d)
+print("Generator function flags", g.__code__.co_flags)
 
 print("Module frame locals", displayDict(sys._getframe().f_locals))
 print("Module flags", sys._getframe().f_code.co_flags)
