@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -179,7 +179,7 @@ try:
 except TypeError as e:
     print(repr(e))
 
-print("Complex call with invalid star list and star arguments:")
+print("Complex call with both invalid star list and star arguments:")
 
 try:
     a = 1
@@ -213,6 +213,8 @@ try:
 except TypeError as e:
     print(repr(e))
 
+print("Complex call with both invalid star list argument:")
+
 try:
     a = 1
 
